;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xb534060393e859bd8654354bad22cc02 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x85cc978d5f9d5b6d497db5681530b87f . "./code.cg2js/cg-gesture-emulation.fasl")
(#x91f26cc5d4ffe987ff5f3591d6d0c38e . "./code.cg2js/cg-grid-widget.fasl")
(#x81bb2484d5a246e009ec9e0e5eccc199 . "./code.cg2js/cg-utility-dialog.fasl")
(#x714f775d5cb81eb1f1bde5cb3f574a12 . "./code.cg2js/cg-wrap-string.fasl")
(#x99b421aa1780d04b93d2e8aa5a64b56a . "./code.cg2js/cg-editable-text.fasl")
(#x86e8b201f4b234698cd45022f9f46001 . "./code.cg2js/cg-base.fasl")
(#x66e8f88b8b3f44247bb26b2bf0a18199 . "./code.cg2js/cg-lisp-text.fasl")
(#x57bd431b4c419ee7ad438f41e8e2c108 . "./code.cg2js/cg-intersect.fasl")
(#x2c3c90172ca2a4289d9b8dae22a73fbf . "./code.cg2js/cg-choose-printer.fasl")
(#x900ba4b5f10c8c3b779d96e30dcdfa00 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x6a99447f5fc00ef3f5c9b42c2e1b41db . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x4c0850564f3ed33bda67c57a907dd729 . "./code.cg2js/cg-font-dialog.fasl")
(#x71c7ee458b3fb7a709bd2535e133ff0d . "./code.cg2js/cg-comtab.fasl")
(#x0654bc93ef2602bb3b8248940cd22bb5 . "./code.cg2js/cg-tooltip.fasl")
(#x16224636855698345e5fdde699f4f28d . "./code.cg2js/cg-chart-or-plot.fasl")
(#x4ec8c7f11cbd40346a3e88a617730845 . "./code.cg2js/cg-get-pixmap.fasl")
(#x0d1fe8d38990a3fde64387deb05606bd . "./code.cg2js/cg-status-bar.fasl")
(#xabbec2449c1228873238296643c62d70 . "./code.cg2js/cg-tab-control.fasl")
(#x8e5c0c7da340c7a7706a9b4a6a815a6d . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd7229d9b762174b089d02e4c5d3a1e87 . "./code.cg2js/cg-list-view.fasl")
(#x1f0985799ea876feff0a2368564d025b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xd756c0dcd0a842b36130f365eaa61370 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x1b63b843e64d78611f4f5ecd05764b44 . "./code.cg2js/cg-project-window.fasl")
(#x5e5a5d5f6c53b8920958cef6b84a3a8f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xe02d7d366cc5ae652e4c5dfd01e4157f . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb301a37f86e303a9fa7aff980cf77f5a . "./code.cg2js/cg-scaling-stream.fasl")
(#xc5e1945a88b35dea7c7fe1334fa112a9 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x004098df9ce1cb647c1491a7e3c550a1 . "./code.cg2js/cg-menu.fasl")
(#x6c1e29863bb3825176cef17000b0d5ed . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd53ab69a850acdddfec8782a70218726 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0793ac22cc747f786fdeab10e8e691ab . "./code.cg2js/cg-split-bar.fasl")
(#x888258aba14b10e29df0b16c62f0bb3f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6a23493986096e1e9135a684865baf1d . "./code.cg2js/cg-timer.fasl")
(#x78f19e0579a2d514b1cf9579b33cb8d4 . "./code.cg2js/cg-text-or-combo.fasl")
(#x0787fa974e4e44e85f08a04089f8f8d5 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xf888b117729a6503608389331c73d162 . "./code.cg2js/cg-string-dialog.fasl")
(#xa9cb2509666900ea4f4ec8667dbb67b5 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x7e3361d5c7fda687c2ba450d7c0ad4df . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa678c2042d3bf8717ee1769e0d8eaa6a . "./code.cg2js/cg-outline.fasl")
(#x0319d880eb9bff0d951e466134e8b75c . "./code.cg2js/cg-lettered-menu.fasl")
(#xeba24c2089e64fcf12228697de0b33c5 . "./code.cg2js/cg-graphics-context.fasl")
(#xe1b109d7881e20625b60919c419c88a6 . "./code.cg2js/cg-item-list.fasl")
(#x01499925a365e2a63bf4756e083b8895 . "./code.cg2js/cg-caret.fasl")
(#x3f800c360fa1edad351440e992966d03 . "./code.cg2js/cg-printing.fasl")
(#x1d3e8fce2db5af8932658916868b55e4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x3d0b8736e9a0f4c2c52480ad1a2daea5 . "./code.cg2js/cg-progress-indicator.fasl")
(#xa0dab80c282e141126d67b74fad2e9a7 . "./code.cg2js/cg-dde-utils.fasl")
(#xb15c8170157ba210f88edb8ea48707f2 . "./code.cg2js/cg-plot-widget.fasl")
(#xe608528cc07045026121e5d5c0504526 . "./code.cg2js/cg-os-window.fasl")
(#x2936ba9dda16c1f90db2023e1531eeeb . "./code.cg2js/cg-calendar.fasl")
(#x73e2c19f8fde81e9f69eed72ff19931b . "./code.cg2js/cg-chart-widget.fasl")
(#x966cdf834f679b505663881ca0db5236 . "./code.cg2js/cg-file-dialog.fasl")
(#xb6c1b6a630c58f9772701c64b7ab950d . "./code.cg2js/cg-nodes.fasl")
(#x7a129ce11eb4087ad2bcdea1bbbe1b32 . "./code.cg2js/cg-lamp.fasl")
(#x4b68c249e0260cd70ec85731d13bee9c . "./code.cg2js/cg-html-dialog.fasl")
(#xc9f0da83dddcdecbac66a904e2343524 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xe7c354d229f22150541e1e1fcc18af1a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x6fa923302ba2c9b9a3a77257d0b888c2 . "./code.cg2js/cg-choice-list.fasl")
(#xc2e6810f77231e6f4b09ce9e9459904c . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7742891eb92235984c40c2e09569c673 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf85f44a98dd8000145348bffdf1f1178 . "./code.cg2js/cg-hotspot.fasl")
(#xd664dafda0f1c1b7624a972571da902d . "./code.cg2js/cg-property.fasl")
(#xdf5efa9d9043c06685497dbf86d8a7d5 . "./code.cg2js/cg-dropping-outline.fasl")
(#xd38ae5d7aeaa3fc6c3a21cb116dd7821 . "./code.cg2js/cg-template-string.fasl")
(#x680f9c5bc0a8b5c9ddf5f854a4063a27 . "./code.cg2js/cg-palette.fasl")
(#x972be13026446e09d08368455f67f8b7 . "./code.cg2js/cg-selected-object.fasl")
(#xec000bed8fa065ef48c7e88c99c9338d . "./code.cg2js/cg-up-down-control.fasl")
(#x5780adec8421991bdf5325bf590af108 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x5ede2b8ea6054764db14966db0aaee89 . "./code.cg2js/cg-layout.fasl")
(#xcf746f63aac0b91d07c01312ed8dab98 . "./code.cg2js/cg-get-position.fasl")
(#xc8de400a5999ca4b6946a4ffdd5a6ac7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x18fd67c84f9e628e1d4d524307cf895a . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1a263ff1e2b701ed1fc3e17310accfc1 . "./code.cg2js/cg-object-editor.fasl")
(#x6f5967c76dd14f9d15659ec6e6805363 . "./code.cg2js/cg-class-grid.fasl")
(#x080365af73075a4b96f0914446f52a00 . "./code.cg2js/cg-yes-no-string.fasl")
(#xd1152ec179e158896900281252d47089 . "./code.cg2js/cg-toggling-widget.fasl")
(#xda83382951df26287166fa7e78f9c609 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x7a8ac23b4b8d99e2834917495c32d39e . "./code.cg2js/cg-button.fasl")
(#x8df0d21295a4beb1c9997d35bb906a66 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x64788e4c4df2e7ebbe0f6415818682dc . "./code.cg2js/cg-scroll-bar.fasl")
(#x8741992e24ec99554587b1baeb5b65cc . "./code.cg2js/cg-combo-box.fasl")
(#xd152fed240b027b24b607c34e90336a5 . "./code.cg2js/cg-web-browser.fasl")
(#x5edeabb5712c99431d3027164a02583a . "./code.cg2js/cg-drawable.fasl")
(#xd21d587c3abc1227636d18c2dfd3ff6a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x85e53685e1c09618be6aa8e82159f790 . "./code.cg2js/cg-message-dialog.fasl")
(#xb78b5fd71030343680bf752305cb215e . "./code.cg2js/cg-text-widget.fasl")
(#x767b9b3435af48ec587af4719771ffe4 . "./code.cg2js/cg-static-text.fasl")
(#x894852181e378f3e48fe7f21b2b67773 . "./code.cg2js/cg-class-support.fasl")
(#x86e1546ad71a35b0fa14e5179d81e3e0 . "./code.cg2js/cg.fasl")
(#x7292ba3a12f6dcf9ecbe0f158c221f70 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5b9db75a6140a8bc241dd587d9685100 . "./code.cg2js/cg-clipboard.fasl")
(#xfcd382c268990839ff77c2f7b8b0414a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x6b707bcd7b5a873a9f579a6251abfff3 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x5b5eb708bdf3a60bf5f752307dd5e801 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd0a0a305b33b917dfdba23f2c198b405 . "./code.cg2js/cg-check-box.fasl")
(#x91731942247e782c4da5b9e251312fd0 . "./code.cg2js/cg-color-dialog.fasl")
(#xdfe91ce9017e9a6e9fea42c55bcb48d8 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x4a20b415270c9a4ac426fc98c70aef98 . "./code.cg2js/cg-dialog-item.fasl")
(#x03686e4102bbdfb3385fadfd275c801b . "./code.cg2js/cg-edit-in-place.fasl")
(#x807aaffec261c56397897277fba4d4b0 . "./code.cg2js/cg-tray.fasl")
(#x1c7d2fc373e31abd6cce4f1623aced96 . "./code.cg2js/cg-paren-matching.fasl")
(#x14b86d5976b4d9635621a5f25b7b7b75 . "./code.cg2js/cg-player.fasl")
(#x043b1503287b1a65ece8eb9f47764971 . "./code.cg2js/cg-yes-no-list.fasl")
(#x1f95e56baa5a0916d02805811a404ae2 . "./code.cg2js/cg-common-control.fasl")
(#x05659650e4e2fb593af0b464264e0feb . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xe724a881f7fff93de76a65e41a822422 . "./code.cg2js/cg-os-widget.fasl")
(#x619a3f19e34c5a466e48de97d56e7bab . "./code.cg2js/cg-sample-file-menu.fasl")
(#x0f90170dca3e280000093615189ca5ad . "./code.cg2js/cg-trackbar.fasl")
(#x7a460a3e829fc76c9cd28e22e0b1d4e7 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x59805538ea7fce9def06405beba803f5 . "./code.cg2js/cg-html-widget.fasl")
(#x94c7d185c7d772c472822313eec9e529 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf1a0fd603bbfc987d056cc5d43e20690 . "./code.cg2js/cg-acache.fasl")
(#x6531e3c6394b0c68a9b488086a4ed553 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x24121dc54c8cc9c60f469b7b6b958dbc . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5e014536411beabae5c3dc3b75d9e11e . "./code.cg2js/cg-icon.fasl")
(#xf0f9d6cb660ea7a2f122fc93677cbb34 . "./code.cg2js/cg-pixmap.fasl")
(#xf4a32dc5df27c00f92088a9afa52bc46 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xe08174c29fd2c61c1cc328b306639e3b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd62c888db955310532367f771236a8b7 . "./code.cg2js/cg-toolbar.fasl")
(#x1d7858140ffbdc79fad21efb0a0d92a9 . "./code.cg2js/cg-radio-button.fasl")
(#xde9a4cca5215bd77297fa8fda18b92a4 . "./code.cg2js/cg-curve.fasl")
(#xe0c4a5d3561fe86776a351bdebdd227b . "./code.cg2js/cg-picture-widget.fasl")
(#x164941e7495bed59cc3e07a045253104 . "./code.cg2js/cg-dev.fasl")
(#xf5967f660367317d0d6c55e2c14630ac . "./code.cg2js/cg-group-box.fasl")
(#x0ddfb8237c91cc0e2efe90598142d0c0 . "./code.cg2js/cg-fill-texture.fasl")
(#xb3684d81a7bd1cde6ab430e327ceb0dd . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x44373ca695712a70c8f41b08756aab91 . "./code.idejs/ide-keyshort.fasl")
(#xf165267074a10e958bc3baecd89a73c3 . "./code.idejs/ide-form.fasl")
(#x58777f327083593320d61a12edb64a6c . "./code.idejs/ide-find-in-files.fasl")
(#x96686688e5844edfcd99cc44a5090902 . "./code.idejs/ide-class-grapher.fasl")
(#x3af055c0ec7d189b2011db1fc74ee71e . "./code.idejs/ide-base.fasl")
(#xa44ed40df51adf57e797f28e388f3b53 . "./code.idejs/ide-class-browser.fasl")
(#xb85f5b35936a087a5c48d356b7fe2233 . "./code.idejs/ide-find-definitions.fasl")
(#xe67de9c95605b12b192ab13f5de7cb3d . "./code.idejs/ide-package-list.fasl")
(#xbea39c64c6b91b11401692de8f2e46a2 . "./code.idejs/ide-grapher.fasl")
(#xa8ed2fb0aa93fb7908a46d6fb67f850f . "./code.idejs/ide-dev.fasl")
(#x619eb3e3234aa075fa1d9415dc94a494 . "./code.idejs/ide.fasl")
(#x16486580ca9a10a25f606aa89fe0d090 . "./code.idejs/ide-trace.fasl")
(#xa48c81b8e86e509ebe17ba504fb9c086 . "./code.idejs/ide-apropos.fasl")
(#xa7db03e5821efa3e7a09af4485b21ad9 . "./code.idejs/ide-inspector.fasl")
(#x58a74c9cd889ff679b5f4bf3a8a6916e . "./code.idejs/ide-acache.fasl")
(#xa374ab71d4cd51b16861bc3eadbf28fa . "./code.idejs/ide-project.fasl")
(#x8d3313e5b069050d0b2a6fb5ab77c213 . "./code.idejs/ide-window-list.fasl")
(#x74cc85e61c061dcba4de390a377078e2 . "./code.idejs/ide-intro.fasl")
(#x9b8971c3d71d913811c834ea21620ff4 . "./code.idejs/ide-clipboard.fasl")
(#x5343c90c65f575407fdca96b06bc4f30 . "./code.idejs/ide-editor.fasl")
(#x0ce6a078280924fb06dddfbd02fbcae8 . "./code.idejs/ide-process-browser.fasl")
(#x6863f41bfda92ae05a1a174f23104ae6 . "./code.idejs/ide-patch-dialog.fasl")
(#x9db1e5e673138d11bd26ccfce1866b01 . "./code.idejs/ide-navigator.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
